#!/usr/bin/env python3
"""
A very lightweight console menu.
"""

__author__ = "Andreas Ehrlund"
__version__ = "0.9.9"
__license__ = "MIT"

import sys

PADDING = 2
MIN_WIDTH = 52


class Menu:

    def __init__(self, menu_title, message):
        self.PADDING = PADDING
        self.MIN_WIDTH = MIN_WIDTH
        self.menu_title = menu_title
        self.message = message
        self.menu_items = []

    def add_function_item(self, title, func_ref, kwargs):
        self.menu_items.append((title, func_ref, kwargs))

    def show(self):
        self.print_menu(self.menu_title, self.message, self.menu_items)

        while True:
            try:
                choice = int(input("Action number: "))
            except ValueError:
                print("Incorrect input, try again.")
                continue

            if 0 == choice:
                break
            elif choice in range(len(self.menu_items) + 1):
                func = self.menu_items[choice - 1][1]
                kwargs = self.menu_items[choice - 1][2]
                func(**kwargs)
                self.show()
                break
            else:
                print("Choose a valid item.")

    def print_menu(self, title, message, menu_items):
        lengths = [len(item[0]) for item in menu_items]
        lengths.extend([len(message), len(title)])
        menu_width = max(self.PADDING + max(lengths), self.MIN_WIDTH)
        if message:
            menu_width = max(menu_width, len(message))
        menu_top_right = (menu_width - len(title) - 1) * "─" + "╮"
        menu_top = f"╭─── {title} {menu_top_right}"
        print(menu_top)

        if message:
            menu_message_right = (menu_width - len(message) + 2) * " " + "│"
            menu_message = f"│ {message} {menu_message_right}"
            print(menu_message)
            print("╞" + (menu_width + 4) * "═" + "╡")

        index = 1
        for item in menu_items:
            print("│ {}: {}{}│".format(
                str(index),
                item[0],
                (menu_width - len(item[0])) * " "
            ))
            index += 1
        print("│ 0: Exit" + (len(menu_top) - 10) * " " + "│")
        print("╰" + (len(menu_top) - 2) * "─" + "╯")



import pytest
from unittest.mock import patch
from unittest.mock import MagicMock
from io import StringIO

from micromenu import micromenu

# pytest --cov=micromenu --cov-report xml:cov.xml tests/


class TestMenu:
    def test_add_function_item(self):
        menu = micromenu.Menu("test", "test1")
        assert len(menu.menu_items) == 0
        menu.add_function_item("title1", lambda x: len(x), {'x': "testparam"})
        assert len(menu.menu_items) == 1

    def test_print_menu(self, capsys):
        menu = micromenu.Menu("test", "test1")
        menu.add_function_item("title1", lambda x: len(x), {'x': "testparam"})
        menu.print_menu(menu.menu_title, menu.message, menu.menu_items)
        captured = capsys.readouterr()
        assert captured.out.startswith("╭─── test ")

    def test_dynamic_width(self, capsys):
        overflow = 4

        # test menu title longest
        menu = micromenu.Menu("X" * (micromenu.MIN_WIDTH + overflow), "x")
        menu.add_function_item("x", lambda x: len(x), {'x': "testparam"})
        menu.print_menu(menu.menu_title, menu.message, menu.menu_items)
        captured = capsys.readouterr()
        lines = captured.out.split('\n')
        del lines[-1]
        assert all(len(x)==len(lines[0]) for x in lines)

        # test menu message longest
        menu = micromenu.Menu("x", "X" * (micromenu.MIN_WIDTH + overflow))
        menu.add_function_item("x", lambda x: len(x), {'x': "testparam"})
        menu.print_menu(menu.menu_title, menu.message, menu.menu_items)
        captured = capsys.readouterr()
        lines = captured.out.split('\n')
        del lines[-1]
        assert all(len(x)==len(lines[0]) for x in lines)

        # test menu item longest
        menu = micromenu.Menu("x", "x")
        menu.add_function_item("X" * (micromenu.MIN_WIDTH + overflow), lambda x: len(x), {'x': "testparam"})
        menu.print_menu(menu.menu_title, menu.message, menu.menu_items)
        captured = capsys.readouterr()
        lines = captured.out.split('\n')
        del lines[-1]
        assert all(len(x)==len(lines[0]) for x in lines)

    def test_invalid_input(self, capsys):
        menu = micromenu.Menu("test", "test1")
        menu.add_function_item("title1", lambda x: len(x), {'x': "testparam"})
        
        with patch('sys.stdin', StringIO("a\n0")):
            menu.show()
        captured = capsys.readouterr()
        assert "Incorrect input, try again." in captured.out
    
    def test_menu_item_out_of_range(self, capsys):
        menu = micromenu.Menu("test", "test1")
        menu.add_function_item("title1", lambda x: len(x), {'x': "testparam"})
        
        with patch('sys.stdin', StringIO("9\n0")):
            menu.show()
        captured = capsys.readouterr()
        assert "Choose a valid item." in captured.out

    def test_menu_item_function_called(self, capsys):
        dummy = MagicMock()

        menu = micromenu.Menu("test", "test1")
        menu.add_function_item("title1", dummy, {'x': "testparam"})
        
        with patch('sys.stdin', StringIO("1\n0")):
            menu.show()

        assert dummy.called





import setuptools

with open("README.md", "r", encoding="utf8") as fh:
    long_description = fh.read()
setuptools.setup(
    name='micromenu',
    version='0.9.9',
    py_modules=['micromenu'],
    author="Andreas Ehrlund",
    author_email="a.ehrlund@gmail.com",
    description="A minimalistic command line menu",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/andli/micromenu",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    extras_require={
        'dev': [
            'wheel',
            'pytest',
            'pytest-cov'
        ],
    },
)
